# Generated by roxygen2: do not edit by hand

S3method(as_xgeo_state,data.frame)
S3method(as_xgeo_state,default)
S3method(as_xgeo_state,matrix)
S3method(as_xgeo_state,xgeo_data)
S3method(as_xgeo_state,xgeo_state)
S3method(print,summary.xgeo_state)
S3method(print,xgeo_state)
S3method(summary,xgeo_state)
export(as_xgeo_state)
export(build_xgeo_lod)
export(compute_xgeo_diagnostics)
export(compute_xgeo_embedding)
export(read_xgeo_state)
export(set_active_embedding)
export(set_xgeo_lod)
export(set_xgeo_selection)
export(validate_xgeo_state)
export(write_xgeo_state)
export(xgeo_attributes)
export(xgeo_explanation_table)
export(xgeo_geometry)
export(xgeo_indices)
export(xgeo_metadata)
export(xgeo_point_values)
export(xgeo_regular_grid)
export(xgeo_selection)
export(xgeo_state)
